STATUS_PARTIAL_CONTENT = 206
STATUS_NOT_SATISFIABLE = 416

Response = namedtuple('Response', ('status', 'headers', 'file'))

class BadRange(Exception):
    pass

def get_range_response(response, range_header):
    content_length = next(v for (k, v) in response.headers
                          if k == 'Content-Length')
    size = int(content_length)
    try:
        start, end = self.get_byte_range(range_header, size)
    except BadRange:
        return Response(self.STATUS_NOT_SATISFIABLE,
                        [('Content-Range', 'bytes */{}'.format(size))],
                        None)
    headers = response.headers
    headers.append(('Content-Range', 'bytes {}-{}/{}'.format(start, end, size)))
    if response.file is not None:
        file_wrapper = self.wrap_file(response.file, start, end, size)
    else:
        file_wrapper = None
    return Response(self.STATUS_PARTIAL_CONTENT, headers, file_wrapper)

def get_btye_range(self, range_header, size):
    raise BadRange()

def wrap_file(self, file_handle, start, end, size):
    if start > 0:
        file_handle.seek(start)
    if end == size:
        return file_handle



"""
Very partial backport of the `http.HTTPStatus` enum from Python 3.5

This implements just enough of the interface for our purposes, it does not
attempt to be a full implementation.
"""
# Python 3.0 - 3.4
try:
    from http import client as status_module
    from http.server import BaseHTTPRequestHandler
    responses = BaseHTTPRequestHandler.responses
# Python 2.7
except ImportError:
    import httplib as status_module
    responses = status_module.responses


class HTTPStatusValue(int):

    phrase = None

    def __new__(cls, code, phrase):
        instance = int.__new__(cls, code)
        instance.phrase = phrase
        return instance


class HTTPStatusProxy(object):

    def __getattr__(self, status):
        code = getattr(status_module, status)
        phrase = responses[code]
        if isinstance(phrase, tuple):
            phrase = phrase[0]
        value = HTTPStatusValue(code, phrase)
        setattr(self, status, value)
        return value


HTTPStatus = HTTPStatusProxy()



from __future__ import unicode_literals

import errno
import os
import stat


class NotARegularFileError(Exception):
    pass


class MissingFileError(NotARegularFileError):
    pass


def stat_regular_file(path):
    """
    Wrap os.stat to raise appropriate errors if `path` is not a regular file
    """
    try:
        file_stat = os.stat(path)
    except OSError as e:
        if e.errno == errno.ENOENT:
            raise MissingFileError(path)
        else:
            raise
    if not stat.S_ISREG(file_stat.st_mode):
        # We ignore directories and treat them as missing files
        if stat.S_ISDIR(file_stat.st_mode):
            raise MissingFileError('Path is a directory: {0}'.format(path))
        else:
            raise NotARegularFileError('Not a regular file: {0}'.format(path))
    return file_stat


def ensure_leading_trailing_slash(path):
    path = (path or '').strip('/')
    return '/{0}/'.format(path) if path else '/'
